/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "connection.h"

#include <string.h>

#include "setting.h"

#include "connection_private.h"
#include "network_private.h"

Connection *connection_new(Network *network, const char *id, const char *name)
{
    Connection *connection = malloc(sizeof(Connection));
    connection->network = network;
    connection->id = (id != NULL) ? strdup(id) : NULL;
    connection->name = (name != NULL) ? strdup(name) : NULL;
    connection->settings = NULL;
    connection->port = NULL;
    return connection;
}

const char *connection_get_uuid(const Connection *connection)
{
    return connection->uuid;
}

const char *connection_get_id(const Connection *connection)
{
    return connection->id;
}

const char *connection_get_name(const Connection *connection)
{
    return connection->name;
}

bool connection_get_autoconnect(const Connection *connection)
{
    return connection->autoconnect;
}

const Settings *connection_get_settings(const Connection *connection)
{
    return connection->settings;
}

const Port *connection_get_port(const Connection *connection)
{
    return connection->port;
}

void connection_add_setting(Connection *connection, Setting *setting)
{
    if (connection->settings == NULL) {
        connection->settings = settings_new(1);
    }
    settings_add(connection->settings, setting);
}

void connection_set_port(Connection *connection, Port *port)
{
    connection->port = port;
}

bool connection_compare(const Connection *c1, const Connection *c2)
{
    return strcmp(c1->id, c2->id) == 0;
}

int connection_delete(const Connection *connection)
{
    return connection_priv_delete(connection);
}

void connection_free(Connection *connection)
{
    if (connection->settings != NULL) {
        settings_free(connection->settings, true);
    }
    if (connection->uuid != NULL) {
        free(connection->uuid);
    }
    if (connection->id != NULL) {
        free(connection->id);
    }
    if (connection->name != NULL) {
        free(connection->name);
    }
    connection_priv_free(connection->priv);
    free(connection);
}

LIST_IMPL(Connection, connection)

Connection *connections_find_by_uuid(const Connections *connections, const char *uuid)
{
    Connection *connection;
    for (size_t i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);
        if (strcmp(connection->uuid, uuid) == 0) {
            return connection;
        }
    }
    return NULL;
}

Connection *connections_find_by_id(const Connections *connections, const char *id)
{
    Connection *connection;
    for (size_t i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);
        if (strcmp(connection->id, id) == 0) {
            return connection;
        }
    }
    return NULL;
}
