/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "network.h"

#include <string.h>
#include <unistd.h>

#include "activeconnection.h"
#include "connection.h"
#include "port.h"

#include "network_private.h"

static Network *_network = NULL;
static pthread_once_t network_is_initialized = PTHREAD_ONCE_INIT;

#ifdef DEBUG_ENABLED

#include <signal.h>
#include <sys/wait.h>

void print_backtrace(int signal)
{
    fprintf(stderr, "BackTrace\n");
    int ppid = getpid();
    int pid = fork();
    if (pid == 0) {
        // Child process
        char *strpid;
        asprintf(&strpid, "%d", ppid);
        execl("/usr/bin/gdb", "/usr/bin/gdb", "-p", strpid, NULL);
    } else {
        int status;
        waitpid(pid, &status, 0);
        system("/usr/bin/pkill -9 sfcbd");
    }
}

#endif

Port *network_port_by_mac(Network *network, const char *mac)
{
    Port *port;
    for (size_t i = 0; i < ports_length(network->ports); ++i) {
        port = ports_index(network->ports, i);
        if (strcmp(port_get_mac(port), mac) == 0) {
            return port;
        }
    }
    return NULL;
}

void *network_thread_start(void *data)
{
    Network *network = data;
    MUTEX_LOCK(network);
    network->priv = network_priv_new(network);
    network->thread_loop = g_main_loop_new(NULL, FALSE);
    MUTEX_UNLOCK(network);

    g_main_loop_run(network->thread_loop);
    g_main_loop_unref(network->thread_loop);
    return NULL;
}

static void network_new()
{
#ifdef DEBUG_ENABLED
    signal(SIGSEGV, print_backtrace);
    signal(SIGABRT, print_backtrace);
    signal(SIGILL, print_backtrace);
    signal(SIGFPE, print_backtrace);
    signal(SIGBUS, print_backtrace);
#endif

    fprintf(stderr, "network_new, pid: %d\n", getpid());
    Network *network;
    network = malloc(sizeof(Network));
    network->loaded = 0;
    network->ref_count = 0;
    network->ports = NULL;
    network->connections = NULL;
    network->activeConnections = NULL;
    pthread_mutex_init(&network->mutex, NULL);
    pthread_create(&network->thread, NULL, network_thread_start, network);
    _network = network;
}

void network_free(Network *network)
{
    g_main_loop_ref(network->thread_loop);
    g_main_loop_quit(network->thread_loop);
    g_main_loop_unref(network->thread_loop);

    void *res = NULL;
    pthread_join(network->thread, &res);
    pthread_mutex_destroy(&network->mutex);
    free(res);
    if (network->priv) {
        network_priv_free(network->priv);
    }
    if (network->ports) {
        ports_free(network->ports, true);
    }
    if (network->connections) {
        connections_free(network->connections, true);
    }
    if (network->activeConnections) {
        active_connections_free(network->activeConnections, true);
    }
    free(network);
    network = NULL;
}

Network *network_ref(const CMPIBroker *broker)
{
    pthread_once(&network_is_initialized, network_new);

    while (true) {
        MUTEX_LOCK(_network);
        if (_network->loaded == (NETWORK_DEVICES_LOADED | NETWORK_SETTINGS_LOADED)) {
            MUTEX_UNLOCK(_network);
            break;
        }
        MUTEX_UNLOCK(_network);
        // We need to wait for other thread to fetch network devices and settings
        // TODO: don't use polling
        usleep(10000);
    }
    MUTEX_LOCK(_network);
    _network->ref_count++;
    MUTEX_UNLOCK(_network);
    return _network;
}

void network_unref(Network *network)
{
    if (network == NULL || _network == NULL) {
        network = _network = NULL;
        return;
    }

    MUTEX_LOCK(network);
    network->ref_count--;

    if (network->ref_count <= 0) {
        network_free(network);
        network = _network = NULL;
    } else {
        MUTEX_UNLOCK(network);
    }
}

void network_lock(Network *network)
{
    MUTEX_LOCK(network);
}

void network_unlock (Network *network)
{
    MUTEX_UNLOCK(network);
}

const Ports *network_get_ports(Network *network)
{
    return network->ports;
}

const Endpoints *network_get_endpoints_by_port(Network *network, Port *port)
{
    return port_get_endpoints(port);
}

const Connections *network_get_connections(Network *network)
{
    return network->connections;
}

int network_activate_connection(Network *network, const Port *port, const Connection *connection)
{
    return network_priv_activate_connection(network, port, connection);
}

const ActiveConnections *network_get_active_connections(Network *network)
{
    return network->activeConnections;
}

int network_create_connection(Network *network, Connection *connection)
{
    return network_priv_create_connection(network, connection);
}

int network_delete_connection(Network *network, Connection *connection)
{
    return network_priv_delete_connection(network, connection);
}
